@@ -1,6 +1,6 @@
 /* pinentry-gtk-2.c
    Copyright (C) 1999 Robert Bihlmeyer <robbe@orcus.priv.at>
-   Copyright (C) 2001, 2002, 2007 g10 Code GmbH
+   Copyright (C) 2001, 2002, 2007, 2015 g10 Code GmbH
    Copyright (C) 2004 by Albrecht Dre <albrecht.dress@arcor.de>
 
    pinentry-gtk-2 is a pinentry application for the Gtk+-2 widget set.
@@ -24,7 +24,15 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+#include <gdk/gdkkeysyms.h>
+#if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 7 )
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wstrict-prototypes"
+#endif
 #include <gtk/gtk.h>
+#if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 7 )
+# pragma GCC diagnostic pop
+#endif
 #include <assert.h>
 #include <math.h>
 #include <stdio.h>
@@ -61,12 +69,9 @@ static GtkWidget *entry;
 static GtkWidget *repeat_entry;
 static GtkWidget *error_label;
 static GtkWidget *qualitybar;
-#ifdef ENABLE_ENHANCED
-static GtkWidget *insure;
-static GtkWidget *time_out;
-#endif
 static GtkTooltips *tooltips;
 static gboolean got_input;
+static guint timeout_source;
 
 /* Gnome hig small and large space in pixels.  */
 #define HIG_SMALL      6
@@ -88,6 +93,8 @@ constrain_size (GtkWidget *win, GtkRequisition *req, gpointer data)
   static gint width, height;
   GdkGeometry geo;
 
+  (void)data;
+
   if (req->width == width && req->height == height)
     return;
   width = req->width;
@@ -121,6 +128,9 @@ make_transient (GtkWidget *win, GdkEvent *event, gpointer data)
   GdkScreen *screen;
   GdkWindow *root;
 
+  (void)event;
+  (void)data;
+
   if (! pinentry->grab)
     return;
 
@@ -135,6 +145,8 @@ make_transient (GtkWidget *win, GdkEvent *event, gpointer data)
 static int
 grab_keyboard (GtkWidget *win, GdkEvent *event, gpointer data)
 {
+  (void)data;
+
   if (! pinentry->grab)
     return FALSE;
 
@@ -152,6 +164,8 @@ grab_keyboard (GtkWidget *win, GdkEvent *event, gpointer data)
 static int
 ungrab_keyboard (GtkWidget *win, GdkEvent *event, gpointer data)
 {
+  (void)data;
+
   gdk_keyboard_ungrab (gdk_event_get_time (event));
   /* Unmake window transient for the root window.  */
   /* gdk_window_set_transient_for cannot be used with parent = NULL to
@@ -166,6 +180,10 @@ ungrab_keyboard (GtkWidget *win, GdkEvent *event, gpointer data)
 static int
 delete_event (GtkWidget *widget, GdkEvent *event, gpointer data)
 {
+  (void)widget;
+  (void)event;
+  (void)data;
+
   pinentry->close_button = 1;
   gtk_main_quit ();
   return TRUE;
@@ -175,21 +193,13 @@ delete_event (GtkWidget *widget, GdkEvent *event, gpointer data)
 static void
 button_clicked (GtkWidget *widget, gpointer data)
 {
+  (void)widget;
+
   if (data)
     {
       const char *s, *s2;
 
       /* Okay button or enter used in text field.  */
-#ifdef ENABLE_ENHANCED
-      /* FIXME: This is not compatible with assuan.  We can't just
-	 print stuff on stdout.  */
-      /* if (pinentry->enhanced) */
-      /*   printf ("Options: %s\nTimeout: %d\n\n", */
-      /*   	gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (insure)) */
-      /*   	? "insure" : "", */
-      /*   	gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (time_out))); */
-#endif
-
       s = gtk_secure_entry_get_text (GTK_SECURE_ENTRY (entry));
       if (!s)
 	s = "";
@@ -223,18 +233,42 @@ button_clicked (GtkWidget *widget, gpointer data)
 static void
 enter_callback (GtkWidget *widget, GtkWidget *anentry)
 {
-  button_clicked (widget, "ok");
+  (void)anentry;
+
+  button_clicked (widget, (gpointer) CONFIRM_OK);
 }
 
 
 static void
 confirm_button_clicked (GtkWidget *widget, gpointer data)
 {
-  confirm_value = (int)(long) data;
+  (void)widget;
+
+  confirm_value = (confirm_value_t) data;
   gtk_main_quit ();
 }
 
 
+static void
+cancel_callback (GtkAccelGroup *acc, GObject *accelerable,
+                 guint keyval, GdkModifierType modifier, gpointer data)
+{
+  int confirm_mode = !!data;
+
+  (void)acc;
+  (void)keyval;
+  (void)modifier;
+
+  if (confirm_mode)
+    confirm_button_clicked (GTK_WIDGET (accelerable),
+                            (gpointer)CONFIRM_CANCEL);
+  else
+    button_clicked (GTK_WIDGET (accelerable),
+                    (gpointer)CONFIRM_CANCEL);
+}
+
+
+
 static gchar *
 pinentry_utf8_validate (gchar *text)
 {
@@ -311,23 +345,39 @@ changed_text_handler (GtkWidget *widget)
 }
 
 
+#ifdef HAVE_LIBSECRET
+static void
+may_save_passphrase_toggled (GtkWidget *widget, gpointer data)
+{
+  GtkToggleButton *button = GTK_TOGGLE_BUTTON (widget);
+  pinentry_t ctx = (pinentry_t) data;
+
+  ctx->may_cache_password = gtk_toggle_button_get_active (button);
+}
+#endif
+
+
 static gboolean
 timeout_cb (gpointer data)
 {
   (void)data;
   if (!got_input)
     gtk_main_quit ();
+
+  /* Don't run again.  */
+  timeout_source = 0;
   return FALSE;
 }
 
 
 static GtkWidget *
-create_window (int confirm_mode)
+create_window (pinentry_t ctx, int confirm_mode)
 {
   GtkWidget *w;
   GtkWidget *win, *box;
   GtkWidget *wvbox, *chbox, *bbox;
   GtkAccelGroup *acc;
+  GClosure *acc_cl;
   gchar *msg;
 
   tooltips = gtk_tooltips_new ();
@@ -498,35 +548,6 @@ create_window (int confirm_mode)
           gtk_widget_show (entry);
           nrow++;
         }
-
-
-#ifdef ENABLE_ENHANCED
-      if (pinentry->enhanced)
-	{
-	  GtkWidget *sbox = gtk_hbox_new (FALSE, HIG_SMALL);
-	  gtk_box_pack_start (GTK_BOX (box), sbox, FALSE, FALSE, 0);
-
-	  w = gtk_label_new ("Forget secret after");
-	  gtk_box_pack_start (GTK_BOX (sbox), w, FALSE, FALSE, 0);
-	  gtk_widget_show (w);
-
-	  time_out = gtk_spin_button_new
-	    (GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, HUGE_VAL, 1, 60, 60)),
-	     2, 0);
-	  gtk_box_pack_start (GTK_BOX (sbox), time_out, FALSE, FALSE, 0);
-	  gtk_widget_show (time_out);
-
-	  w = gtk_label_new ("seconds");
-	  gtk_box_pack_start (GTK_BOX (sbox), w, FALSE, FALSE, 0);
-	  gtk_widget_show (w);
-	  gtk_widget_show (sbox);
-
-	  insure = gtk_check_button_new_with_label ("ask before giving out "
-						    "secret");
-	  gtk_box_pack_start (GTK_BOX (box), insure, FALSE, FALSE, 0);
-	  gtk_widget_show (insure);
-	}
-#endif
     }
 
   bbox = gtk_hbutton_box_new ();
@@ -534,6 +555,32 @@ create_window (int confirm_mode)
   gtk_box_set_spacing (GTK_BOX (bbox), 6);
   gtk_box_pack_start (GTK_BOX (wvbox), bbox, TRUE, FALSE, 0);
 
+#ifdef HAVE_LIBSECRET
+  if (ctx->allow_external_password_cache && ctx->keyinfo)
+    /* Only show this if we can cache passwords and we have a stable
+       key identifier.  */
+    {
+      if (pinentry->default_pwmngr)
+        {
+          msg = pinentry_utf8_validate (pinentry->default_pwmngr);
+          w = gtk_check_button_new_with_mnemonic (msg);
+          g_free (msg);
+        }
+      else
+        w = gtk_check_button_new_with_label ("Save passphrase using libsecret");
+
+      /* Make sure it is off by default.  */
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), FALSE);
+
+      gtk_box_pack_start (GTK_BOX (box), w, TRUE, FALSE, 0);
+      gtk_widget_show (w);
+
+      g_signal_connect (G_OBJECT (w), "toggled",
+                        G_CALLBACK (may_save_passphrase_toggled),
+			(gpointer) ctx);
+    }
+#endif
+
   if (!pinentry->one_button)
     {
       if (pinentry->cancel)
@@ -561,6 +608,11 @@ create_window (int confirm_mode)
                         G_CALLBACK (confirm_mode ? confirm_button_clicked
                                     : button_clicked),
 			(gpointer) CONFIRM_CANCEL);
+
+      acc_cl = g_cclosure_new (G_CALLBACK (cancel_callback),
+			       (confirm_mode? "":NULL), NULL);
+      gtk_accel_group_connect (acc, GDK_KEY_Escape, 0, 0, acc_cl);
+
       GTK_WIDGET_SET_FLAGS (w, GTK_CAN_DEFAULT);
     }
 
@@ -622,7 +674,7 @@ create_window (int confirm_mode)
   gtk_window_present (GTK_WINDOW (win));  /* Make sure it has the focus.  */
 
   if (pinentry->timeout > 0)
-    g_timeout_add (pinentry->timeout*1000, timeout_cb, NULL);
+    timeout_source = g_timeout_add (pinentry->timeout*1000, timeout_cb, NULL);
 
   return win;
 }
@@ -638,12 +690,19 @@ gtk_cmd_handler (pinentry_t pe)
   pinentry = pe;
   confirm_value = CONFIRM_CANCEL;
   passphrase_ok = 0;
-  w = create_window (want_pass ? 0 : 1);
+  w = create_window (pe, want_pass ? 0 : 1);
   gtk_main ();
   gtk_widget_destroy (w);
   while (gtk_events_pending ())
     gtk_main_iteration ();
 
+  if (timeout_source)
+    /* There is a timer running.  Cancel it.  */
+    {
+      g_source_remove (timeout_source);
+      timeout_source = 0;
+    }
+
   if (confirm_value == CONFIRM_CANCEL || grab_failed)
     pe->canceled = 1;
 
@@ -682,19 +741,17 @@ main (int argc, char *argv[])
 
 #ifdef FALLBACK_CURSES
   if (pinentry_have_display (argc, argv))
-    gtk_init (&argc, &argv);
+    {
+      if (! gtk_init_check (&argc, &argv))
+	pinentry_cmd_handler = curses_cmd_handler;
+    }
   else
     pinentry_cmd_handler = curses_cmd_handler;
 #else
   gtk_init (&argc, &argv);
 #endif
 
-  /* Consumes all arguments.  */
-  if (pinentry_parse_opts (argc, argv))
-    {
-      printf(PGMNAME " " VERSION "\n");
-      exit(EXIT_SUCCESS);
-    }
+  pinentry_parse_opts (argc, argv);
 
   if (pinentry_loop ())
     return 1;
